/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : EdgeGraph.cc                                             */
/*                                                                       */
/*       Date : November  2, 1997                                        */
/* Updated at : November  2, 1997                                        */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/


#include "AirScheduleProblem.h"
#include "EdgeGraph.h"
#include <iomanip.h>
#include <iostream.h>
#include <stdio.h>

/*
//
*/

EdgeGraph::EdgeGraph(unsigned _size)
{

   size = _size;
   matrix = new float[size*size];

}// end of CONSTRUCTOR

/*
//
*/

EdgeGraph::EdgeGraph()
{
  size = 0;
  matrix = NULL;

}// end of CONSTRUCTOR

/*
//
*/

EdgeGraph::EdgeGraph(const EdgeGraph& RHS)
{
   copyFrom(RHS);

}// end of COPY CONSTRUCTOR

/*
//
*/

EdgeGraph::~EdgeGraph() 
{
   dispose();

}// end of Destructor

/*
//
*/

EdgeGraph::buildGraphFor(AirScheduleProblem *prob)
{
   int i, j;
   int retVal= 0;
   float threshold = prob->getMinimumTime();
   Flight *flightI = NULL;
   Flight *flightJ = NULL;

   if(size==prob->getNumberOfFlights())
   {
      for(i=0;i<size;i++)
          for(j=0;j<size;j++) matrix[i*size+j] = 0;

      for(i=0;i<(size-1);i++)
      {
	 flightI = prob->getFlightWithIndex(i);

	 for(j=i+1;j<size;j++)
         {
	    flightJ = prob->getFlightWithIndex(j);

	    if( flightI->getToIndex() == flightJ->getFromIndex() &&
		(flightJ->getDepartureDate()-flightI->getArrivalDate())>threshold)
		   matrix[i*size+j] = flightJ->getDepartureDate()-flightI->getArrivalDate();
	     
         }// end of for

      }// end of for

   }// end of if

   return retVal;

}// end of buildGraphFor()

/*
//
*/

float
EdgeGraph::getCost(unsigned from, unsigned to) const
{
   float retVal = 0;

   if( from>=0 && to>=0 && from<size && to <size )
       retVal = matrix[from*size+to];

   return retVal;

}// end of getCost()

/*
//
*/

void
EdgeGraph::nicePrint(ostream& os) const
{
  int i, j, k=0;

  for(i=0;i<size;i++)
  {   
      os << setw(3) << i ;

      for(j=0;j<size;j++)
	 os << setw(2) << (matrix[i*size+j]>0?1:0);

      os << endl;
  }

  os << endl;

  for(i=0;i<size;i++)
  {   
      for(j=0;j<size;j++)
      {   
	 if(matrix[i*size+j]>0)	 
         {   os<<" m["<<setw(2)<<i<<","<<setw(2)<<j<<"] = "<< setw(6) << matrix[i*size+j];
	     k++;
	     if(k%5==0) os << endl; 
	 }
      }
  }
   
   

}// end of nicePrint()

/*
//
*/

EdgeGraph::operator==(const EdgeGraph& RHS)
{
   int retVal = 0;
   int i, j;

   retVal = (size == RHS.size);

   for(i=0;retVal && i<size;i++)
      for(j=0;retVal && j<size;j++)
	  retVal = ( matrix[i*size+j] == RHS.matrix[i*size+j]);

   return retVal;

}// end of operator==()

/*
//
*/

EdgeGraph
EdgeGraph::operator=(const EdgeGraph& RHS)
{
   dispose();
   copyFrom(RHS);

   return (*this);

}

void 
EdgeGraph::dispose()
{
   if(size>0) { size=0; delete[] matrix; }

}// end of dispose()

void
EdgeGraph::copyFrom(const EdgeGraph& RHS)
{
   int i, j; 

   size = RHS.size;
   matrix = new float[size*size];
   for(i=0;i<size;i++)
       for(j=0;j<size;j++)
	  matrix[i*size+j] = RHS.matrix[i*size+j];

}// end of copyFrom()

/*
//
*/

ostream& 
operator<<(ostream& os,const EdgeGraph& RHS)
{
   RHS.nicePrint(os);
   return os;

}// end of operator<<()

